@@ -32,19 +32,23 @@ use uv_configuration::RAYON_INITIALIZE;
 use uv_normalize::PackageName;
 use uv_python::PythonEnvironment;
 
+use crate::common::pypi_index::PyPIPackage;
 use crate::common::{
     LockFileExt, PixiControl,
     builders::{
         HasDependencyConfig, HasLockFileUpdateConfig, HasNoInstallConfig, string_from_iter,
     },
-    package_database::{Package, PackageDatabase},
 };
 use crate::setup_tracing;
+use pixi_test_utils::{MockRepoData, Package};
 
 /// Should add a python version to the environment and lock file that matches
 /// the specified version and run it
 #[tokio::test]
-#[cfg_attr(not(feature = "slow_integration_tests"), ignore)]
+#[cfg_attr(
+    any(not(feature = "online_tests"), not(feature = "slow_integration_tests")),
+    ignore
+)]
 async fn install_run_python() {
     setup_tracing();
 
@@ -97,7 +101,7 @@ async fn install_run_python() {
 async fn test_incremental_lock_file() {
     setup_tracing();
 
-    let mut package_database = PackageDatabase::default();
+    let mut package_database = MockRepoData::default();
 
     // Add a package `foo` that depends on `bar` both set to version 1.
     package_database.add_package(Package::build("bar", "1").finish());
@@ -175,7 +179,10 @@ async fn test_incremental_lock_file() {
 
 /// Test the `pixi install --locked` functionality.
 #[tokio::test]
-#[cfg_attr(not(feature = "slow_integration_tests"), ignore)]
+#[cfg_attr(
+    any(not(feature = "online_tests"), not(feature = "slow_integration_tests")),
+    ignore
+)]
 async fn install_locked_with_config() {
     setup_tracing();
 
@@ -278,7 +285,10 @@ async fn install_locked_with_config() {
 
 /// Test `pixi install/run --frozen` functionality
 #[tokio::test]
-#[cfg_attr(not(feature = "slow_integration_tests"), ignore)]
+#[cfg_attr(
+    any(not(feature = "online_tests"), not(feature = "slow_integration_tests")),
+    ignore
+)]
 async fn install_frozen() {
     setup_tracing();
 
@@ -357,7 +367,10 @@ async fn is_conda_package_installed(prefix_path: &Path, package_name: &str) -> b
 
 /// Test `pixi install --frozen --skip` functionality
 #[tokio::test]
-#[cfg_attr(not(feature = "slow_integration_tests"), ignore)]
+#[cfg_attr(
+    any(not(feature = "online_tests"), not(feature = "slow_integration_tests")),
+    ignore
+)]
 async fn install_frozen_skip() {
     setup_tracing();
 
@@ -422,7 +435,10 @@ async fn install_frozen_skip() {
 }
 
 #[tokio::test(flavor = "multi_thread", worker_threads = 1)]
-#[cfg_attr(not(feature = "slow_integration_tests"), ignore)]
+#[cfg_attr(
+    any(not(feature = "online_tests"), not(feature = "slow_integration_tests")),
+    ignore
+)]
 async fn pypi_reinstall_python() {
     setup_tracing();
 
@@ -479,7 +495,10 @@ async fn pypi_reinstall_python() {
 }
 
 #[tokio::test(flavor = "multi_thread", worker_threads = 1)]
-#[cfg_attr(not(feature = "slow_integration_tests"), ignore)]
+#[cfg_attr(
+    any(not(feature = "online_tests"), not(feature = "slow_integration_tests")),
+    ignore
+)]
 // Check if we add and remove a pypi package that the site-packages is cleared
 async fn pypi_add_remove() {
     setup_tracing();
@@ -520,13 +539,13 @@ async fn test_channels_changed() {
     setup_tracing();
 
     // Write a channel with a package `bar` with only one version
-    let mut package_database_a = PackageDatabase::default();
+    let mut package_database_a = MockRepoData::default();
     package_database_a.add_package(Package::build("bar", "2").finish());
     let channel_a = package_database_a.into_channel().await.unwrap();
 
     // Write another channel with a package `bar` with only one version but another
     // one.
-    let mut package_database_b = PackageDatabase::default();
+    let mut package_database_b = MockRepoData::default();
     package_database_b.add_package(Package::build("bar", "1").finish());
     let channel_b = package_database_b.into_channel().await.unwrap();
 
@@ -587,7 +606,10 @@ async fn install_conda_meta_history() {
 }
 
 #[tokio::test(flavor = "multi_thread", worker_threads = 1)]
-#[cfg_attr(not(feature = "slow_integration_tests"), ignore)]
+#[cfg_attr(
+    any(not(feature = "online_tests"), not(feature = "slow_integration_tests")),
+    ignore
+)]
 async fn minimal_lockfile_update_pypi() {
     setup_tracing();
 
@@ -633,7 +655,10 @@ async fn minimal_lockfile_update_pypi() {
 /// then change the installer back and see if it reinstalls the package
 /// with a new version
 #[tokio::test(flavor = "multi_thread", worker_threads = 1)]
-#[cfg_attr(not(feature = "slow_integration_tests"), ignore)]
+#[cfg_attr(
+    any(not(feature = "online_tests"), not(feature = "slow_integration_tests")),
+    ignore
+)]
 async fn test_installer_name() {
     setup_tracing();
 
@@ -731,7 +756,10 @@ async fn test_old_lock_install() {
 }
 
 #[tokio::test(flavor = "multi_thread", worker_threads = 1)]
-#[cfg_attr(not(feature = "slow_integration_tests"), ignore)]
+#[cfg_attr(
+    any(not(feature = "online_tests"), not(feature = "slow_integration_tests")),
+    ignore
+)]
 async fn test_no_build_isolation() {
     setup_tracing();
 
@@ -815,7 +843,10 @@ setup(
 }
 
 #[tokio::test(flavor = "multi_thread", worker_threads = 1)]
-#[cfg_attr(not(feature = "slow_integration_tests"), ignore)]
+#[cfg_attr(
+    any(not(feature = "online_tests"), not(feature = "slow_integration_tests")),
+    ignore
+)]
 async fn test_no_build_isolation_with_dependencies() {
     setup_tracing();
 
@@ -938,7 +969,10 @@ setup(
 }
 
 #[tokio::test(flavor = "multi_thread", worker_threads = 1)]
-#[cfg_attr(not(feature = "slow_integration_tests"), ignore)]
+#[cfg_attr(
+    any(not(feature = "online_tests"), not(feature = "slow_integration_tests")),
+    ignore
+)]
 async fn test_setuptools_override_failure() {
     setup_tracing();
 
@@ -982,7 +1016,10 @@ async fn test_setuptools_override_failure() {
 }
 
 #[tokio::test(flavor = "multi_thread", worker_threads = 1)]
-#[cfg_attr(not(feature = "slow_integration_tests"), ignore)]
+#[cfg_attr(
+    any(not(feature = "online_tests"), not(feature = "slow_integration_tests")),
+    ignore
+)]
 async fn test_many_linux_wheel_tag() {
     setup_tracing();
 
@@ -1001,7 +1038,6 @@ async fn test_many_linux_wheel_tag() {
     // We know that this package has many linux wheel tags for this version
     pixi.add("gmsh==4.13.1")
         .set_type(pixi_core::DependencyType::PypiDependency)
-        .with_install(true)
         .await
         .unwrap();
 }
@@ -1061,7 +1097,10 @@ async fn test_ensure_gitignore_file_creation() {
 }
 
 #[tokio::test(flavor = "multi_thread", worker_threads = 1)]
-#[cfg_attr(not(feature = "slow_integration_tests"), ignore)]
+#[cfg_attr(
+    any(not(feature = "online_tests"), not(feature = "slow_integration_tests")),
+    ignore
+)]
 async fn pypi_prefix_is_not_created_when_whl() {
     setup_tracing();
 
@@ -1102,10 +1141,30 @@ async fn pypi_prefix_is_not_created_when_whl() {
 /// This should result in the PyPI package being overridden on linux and not on
 /// osxarm64.
 #[tokio::test]
-#[cfg_attr(not(feature = "slow_integration_tests"), ignore)]
 async fn conda_pypi_override_correct_per_platform() {
     setup_tracing();
 
+    // Create local conda channel with Python for multiple platforms
+    let mut package_db = MockRepoData::default();
+    package_db.add_package(
+        Package::build("python", "3.12.0")
+            .with_subdir(Platform::NoArch)
+            .finish(),
+    );
+    package_db.add_package(
+        Package::build("boltons", "1.0.0")
+            .with_subdir(Platform::NoArch)
+            .with_pypi_purl("boltons")
+            .finish(),
+    );
+    let channel = package_db.into_channel().await.unwrap();
+
+    // Create local PyPI index with test packages
+    let pypi_index = crate::common::pypi_index::Database::new()
+        .with(PyPIPackage::new("boltons", "1.0.0"))
+        .into_simple_index()
+        .unwrap();
+
     let pixi = PixiControl::new().unwrap();
     pixi.init_with_platforms(vec![
         Platform::OsxArm64.to_string(),
@@ -1113,10 +1172,20 @@ async fn conda_pypi_override_correct_per_platform() {
         Platform::Win64.to_string(),
         Platform::Osx64.to_string(),
     ])
+    .with_local_channel(channel.url().to_file_path().unwrap())
     .await
     .unwrap();
     pixi.add("python==3.12").with_install(false).await.unwrap();
 
+    // Add pypi-options to the manifest
+    let manifest = pixi.manifest_contents().unwrap();
+    let updated_manifest = format!(
+        "{}\n[pypi-options]\nindex-url = \"{}\"\n",
+        manifest,
+        pypi_index.index_url()
+    );
+    pixi.update_manifest(&updated_manifest).unwrap();
+
     // Add a conda package that is only available on linux
     pixi.add("boltons")
         .with_platform(Platform::Linux64)
@@ -1158,8 +1227,10 @@ async fn conda_pypi_override_correct_per_platform() {
 }
 
 #[tokio::test(flavor = "multi_thread", worker_threads = 2)]
-#[cfg_attr(not(feature = "slow_integration_tests"), ignore)]
-
+#[cfg_attr(
+    any(not(feature = "online_tests"), not(feature = "slow_integration_tests")),
+    ignore
+)]
 async fn test_multiple_prefix_update() {
     setup_tracing();
 
@@ -1387,7 +1458,7 @@ async fn install_s3() {
 async fn test_exclude_newer() {
     setup_tracing();
 
-    let mut package_database = PackageDatabase::default();
+    let mut package_database = MockRepoData::default();
 
     // Create a channel with two packages with different timestamps
     package_database.add_package(
@@ -1456,7 +1527,10 @@ async fn test_exclude_newer() {
 }
 
 #[tokio::test]
-#[cfg_attr(not(feature = "slow_integration_tests"), ignore)]
+#[cfg_attr(
+    any(not(feature = "online_tests"), not(feature = "slow_integration_tests")),
+    ignore
+)]
 async fn test_exclude_newer_pypi() {
     setup_tracing();
 
