@@ -15,7 +15,7 @@ use pixi_build_frontend::{
     tool::{IsolatedTool, SystemTool, Tool},
 };
 use pixi_build_types::{
-    PixiBuildApiVersion, ProjectModelV1, TargetSelectorV1, procedures::initialize::InitializeParams,
+    PixiBuildApiVersion, ProjectModel, TargetSelector, procedures::initialize::InitializeParams,
 };
 use pixi_path::{AbsPresumedDirPathBuf, AbsPresumedFilePathBuf};
 use pixi_spec::SpecConversionError;
@@ -25,6 +25,7 @@ use rattler_shell::{
     shell::ShellEnum,
 };
 use rattler_virtual_packages::DetectVirtualPackageError;
+use std::sync::Arc;
 use thiserror::Error;
 
 #[derive(Debug)]
@@ -39,13 +40,13 @@ pub struct InstantiateBackendSpec {
     pub manifest_path: AbsPresumedFilePathBuf,
 
     /// Optionally, the manifest of the discovered package.
-    pub project_model: Option<ProjectModelV1>,
+    pub project_model: Option<ProjectModel>,
 
     /// Additional configuration that applies to the backend.
     pub configuration: Option<serde_json::Value>,
 
     /// Targets that apply to the backend.
-    pub target_configuration: Option<OrderMap<TargetSelectorV1, serde_json::Value>>,
+    pub target_configuration: Option<OrderMap<TargetSelector, serde_json::Value>>,
 
     /// The source directory to use for the backend
     pub build_source_dir: AbsPresumedDirPathBuf,
@@ -69,13 +70,13 @@ impl CommandDispatcher {
         // Canonicalize the source_dir to ensure it's a fully resolved absolute path
         // without any relative components like ".." or "."
         let source_dir = dunce::canonicalize(&spec.build_source_dir).map_err(|e| {
-            CommandDispatcherError::Failed(InstantiateBackendError::SpecConversionError(
+            CommandDispatcherError::Failed(InstantiateBackendError::SpecConversionError(Arc::new(
                 SpecConversionError::InvalidPath(format!(
                     "failed to canonicalize source directory '{}': {}",
                     spec.build_source_dir.display(),
                     e
                 )),
-            ))
+            )))
         })?;
 
         let command_spec = match self.build_backend_overrides() {
@@ -89,14 +90,14 @@ impl CommandDispatcher {
                     let memory = in_mem
                         .initialize(InitializeParams {
                             manifest_path: spec.manifest_path.to_std_path_buf(),
-                            source_dir: Some(source_dir),
-                            workspace_root: Some(spec.workspace_root.to_std_path_buf()),
+                            source_directory: Some(source_dir),
+                            workspace_directory: Some(spec.workspace_root.to_std_path_buf()),
                             cache_directory: Some(self.cache_dirs().root().to_owned().into()),
-                            project_model: spec.project_model.map(Into::into),
+                            project_model: spec.project_model,
                             configuration: spec.configuration,
                             target_configuration: spec.target_configuration,
                         })
-                        .map_err(InstantiateBackendError::InMemoryError)
+                        .map_err(InstantiateBackendError::from)
                         .map_err(CommandDispatcherError::Failed)?;
                     return Ok(Backend::new(memory.into(), in_mem.api_version()));
                 } else {
@@ -165,10 +166,15 @@ impl CommandDispatcher {
 
         // Add debug information about what the backend supports.
         tracing::info!(
-            "Instantiated backend {}{}, negotiated API version {}",
+            "Instantiated backend {}{}, negotiated API version {}{}",
             tool.executable(),
             tool.version().map_or_else(String::new, |v| format!("@{v}")),
             api_version,
+            if let Some(isolated_tool) = tool.as_isolated() {
+                format!(", from prefix {}", isolated_tool.prefix().display())
+            } else {
+                "".to_string()
+            },
         );
 
         // Make sure that the project model is compatible with the API version.
@@ -179,7 +185,9 @@ impl CommandDispatcher {
                 .is_some_and(|p| p.name.is_none())
         {
             return Err(CommandDispatcherError::Failed(
-                InstantiateBackendError::SpecConversionError(SpecConversionError::MissingName),
+                InstantiateBackendError::SpecConversionError(Arc::new(
+                    SpecConversionError::MissingName,
+                )),
             ));
         }
 
@@ -200,33 +208,63 @@ impl CommandDispatcher {
     }
 }
 
-#[derive(Debug, Error, Diagnostic)]
+#[derive(Debug, Clone, Error, Diagnostic)]
 pub enum InstantiateBackendError {
     /// The command dispatcher could not be initialized.
     #[error(transparent)]
     #[diagnostic(transparent)]
-    JsonRpc(#[from] json_rpc::InitializeError),
+    JsonRpc(Arc<json_rpc::InitializeError>),
 
     /// The command dispatcher could not be initialized.
     #[error(transparent)]
     #[diagnostic(transparent)]
-    InMemoryError(Box<CommunicationError>),
+    InMemoryError(Arc<CommunicationError>),
 
     /// Could not detect the virtual packages for the system
     #[error(transparent)]
-    VirtualPackages(#[from] DetectVirtualPackageError),
+    VirtualPackages(Arc<DetectVirtualPackageError>),
 
     #[error(transparent)]
     #[diagnostic(transparent)]
     InstantiateToolEnvironment(#[from] InstantiateToolEnvironmentError),
 
     #[error("failed to run activation for the backend tool")]
-    Activation(#[from] ActivationError),
+    Activation(Arc<ActivationError>),
 
     #[error(transparent)]
-    SpecConversionError(#[from] SpecConversionError),
+    SpecConversionError(Arc<SpecConversionError>),
 
     #[error(transparent)]
     #[diagnostic(transparent)]
     SourceCheckout(#[from] SourceCheckoutError),
 }
+
+impl From<json_rpc::InitializeError> for InstantiateBackendError {
+    fn from(err: json_rpc::InitializeError) -> Self {
+        Self::JsonRpc(Arc::new(err))
+    }
+}
+
+impl From<Box<CommunicationError>> for InstantiateBackendError {
+    fn from(err: Box<CommunicationError>) -> Self {
+        Self::InMemoryError(Arc::new(*err))
+    }
+}
+
+impl From<DetectVirtualPackageError> for InstantiateBackendError {
+    fn from(err: DetectVirtualPackageError) -> Self {
+        Self::VirtualPackages(Arc::new(err))
+    }
+}
+
+impl From<ActivationError> for InstantiateBackendError {
+    fn from(err: ActivationError) -> Self {
+        Self::Activation(Arc::new(err))
+    }
+}
+
+impl From<SpecConversionError> for InstantiateBackendError {
+    fn from(err: SpecConversionError) -> Self {
+        Self::SpecConversionError(Arc::new(err))
+    }
+}
