@@ -22,7 +22,7 @@ use pixi_command_dispatcher::{
 };
 use pixi_config::default_channel_config;
 use pixi_record::{PinnedPathSpec, PinnedSourceSpec};
-use pixi_spec::{GitReference, GitSpec, PathSpec, PixiSpec, UrlSpec};
+use pixi_spec::{GitReference, GitSpec, PathSpec, PixiSpec, Subdirectory, UrlSpec};
 use pixi_spec_containers::DependencyMap;
 use pixi_test_utils::format_diagnostic;
 use pixi_url::UrlError;
@@ -137,7 +137,7 @@ pub async fn simple_test() {
                 GitSpec {
                     git: git_repo.url.parse().unwrap(),
                     rev: Some(GitReference::Rev(git_repo.commits[0].clone())),
-                    subdirectory: Some(String::from("recipe")),
+                    subdirectory: Subdirectory::try_from("recipe").unwrap(),
                 }
                 .into(),
             )]),
@@ -266,7 +266,7 @@ pub async fn instantiate_backend_without_compatible_api_version_cancels_duplicat
         dispatcher.instantiate_tool_environment(spec),
     );
 
-    // Exactly one result should be a failure and the other should be cancelled.
+    // Both results should be failures since errors are now cloned and sent to all channels.
     let is_cancelled = |r: &Result<
         _,
         CommandDispatcherError<pixi_command_dispatcher::InstantiateToolEnvironmentError>,
@@ -280,10 +280,13 @@ pub async fn instantiate_backend_without_compatible_api_version_cancels_duplicat
     let failed_count = usize::from(is_failed(&r1)) + usize::from(is_failed(&r2));
 
     assert_eq!(
-        cancelled_count, 1,
-        "expected exactly one request to be cancelled"
+        cancelled_count, 0,
+        "no requests should be cancelled - errors are cloned to all channels"
+    );
+    assert_eq!(
+        failed_count, 2,
+        "both requests should fail with the same error"
     );
-    assert_eq!(failed_count, 1, "expected exactly one request to fail");
 }
 
 /// Dropping the returned future should cancel the background task promptly.
@@ -1150,6 +1153,7 @@ pub async fn pin_and_checkout_url_reuses_cached_checkout() {
         url: "https://example.com/archive.tar.gz".parse().unwrap(),
         md5: None,
         sha256: Some(sha),
+        subdirectory: Subdirectory::default(),
     };
 
     let checkout = dispatcher
@@ -1183,11 +1187,13 @@ pub async fn pin_and_checkout_url_reports_sha_mismatch_from_concurrent_request()
         url: url.clone(),
         md5: None,
         sha256: None,
+        subdirectory: Subdirectory::default(),
     };
     let bad_spec = UrlSpec {
         url,
         md5: None,
         sha256: Some(Sha256::digest(b"pixi-url-bad-sha")),
+        subdirectory: Subdirectory::default(),
     };
 
     let (good, bad) = tokio::join!(
@@ -1220,6 +1226,7 @@ pub async fn pin_and_checkout_url_validates_cached_results() {
         url: url.clone(),
         md5: None,
         sha256: None,
+        subdirectory: Subdirectory::default(),
     };
 
     dispatcher
@@ -1231,6 +1238,7 @@ pub async fn pin_and_checkout_url_validates_cached_results() {
         url: url.clone(),
         md5: None,
         sha256: Some(Sha256::digest(b"pixi-url-bad-cache")),
+        subdirectory: Subdirectory::default(),
     };
 
     let err = dispatcher.checkout_url(bad_spec).await.unwrap_err();
