@@ -219,6 +219,9 @@ impl<'de> toml_span::Deserialize<'de> for TomlTask {
             let env = th
                 .optional::<TomlIndexMap<_, _>>("env")
                 .map(TomlIndexMap::into_inner);
+            let default_environment = th
+                .optional::<TomlFromStr<EnvironmentName>>("default-environment")
+                .map(TomlFromStr::into_inner);
             let description = th.optional("description");
             let clean_env = th.optional("clean-env").unwrap_or(false);
             let args = th.optional::<Vec<TaskArg>>("args");
@@ -247,6 +250,7 @@ impl<'de> toml_span::Deserialize<'de> for TomlTask {
                 depends_on,
                 cwd,
                 env,
+                default_environment,
                 description,
                 clean_env,
                 args,
