@@ -1,7 +1,10 @@
-mod path_utils;
+mod dev_source_record;
 mod pinned_source;
 mod source_record;
 
+pub use dev_source_record::DevSourceRecord;
+mod path_utils;
+
 use std::path::Path;
 
 pub use pinned_source::{
@@ -14,7 +17,7 @@ use rattler_conda_types::{
 };
 use rattler_lock::{CondaPackageData, ConversionError, UrlOrPath};
 use serde::Serialize;
-pub use source_record::{InputHash, SourceRecord};
+pub use source_record::SourceRecord;
 use thiserror::Error;
 
 /// A record of a conda package that is either something installable from a
