@@ -217,10 +217,7 @@ impl BuildDownloadVerifyReporter {
             .get_mut(&index)
             .expect("entry is missing from tracker");
         let EntryState::Validating = entry.state else {
-            panic!(
-                "Expected entry to be in downloading state, actual: {:?}",
-                entry
-            );
+            panic!("Expected entry to be in downloading state, actual: {entry:?}");
         };
         entry.state = EntryState::Finished { download: None };
         drop(entries);
