@@ -1,8 +1,9 @@
+use pixi_consts::consts::{KNOWN_MANIFEST_FILES, RATTLER_BUILD_FILE_NAMES, ROS_BACKEND_FILE_NAMES};
 use typed_path::{
     Utf8Component, Utf8Encoding, Utf8Path, Utf8PathBuf, Utf8TypedPath, Utf8TypedPathBuf,
 };
 
-use crate::{GitSpec, PathSourceSpec, SourceLocationSpec, SourceSpec, UrlSourceSpec};
+use crate::{GitSpec, PathSourceSpec, SourceLocationSpec, SourceSpec, Subdirectory, UrlSourceSpec};
 
 /// `SourceAnchor` represents the resolved base location of a `SourceSpec`.
 /// It serves as a reference point for interpreting relative or recursive
@@ -58,7 +59,20 @@ impl SourceAnchor {
 
         match base {
             SourceLocationSpec::Path(PathSourceSpec { path: base }) => {
-                let relative_path = normalize_typed(base.join(path).to_path());
+                // Use the parent directory as the base when the base path points to
+                // a manifest file (e.g., `package-a/pixi.toml` -> `package-a`).
+                // This ensures relative paths like `../package-b` resolve correctly.
+                //
+                // We check for known manifest file names rather than using filesystem
+                // checks because the path may be relative to the workspace root,
+                // not the current working directory.
+                let base_dir = if is_known_manifest_file(base.to_path()) {
+                    base.parent().unwrap_or_else(|| base.to_path())
+                } else {
+                    base.to_path()
+                };
+
+                let relative_path = normalize_typed(base_dir.join(path).to_path());
                 SourceLocationSpec::Path(PathSourceSpec {
                     path: relative_path,
                 })
@@ -71,21 +85,45 @@ impl SourceAnchor {
                 rev,
                 subdirectory,
             }) => {
+                let base_subdir = subdirectory.as_path().to_string_lossy();
                 let relative_subdir = normalize_typed(
-                    Utf8TypedPath::from(subdirectory.as_deref().unwrap_or_default())
+                    Utf8TypedPath::from(base_subdir.as_ref())
                         .join(path)
                         .to_path(),
                 );
+                // Convert to Subdirectory (defaults to empty if normalization results in empty)
+                let subdir_str = relative_subdir.to_string();
+                let subdirectory = if subdir_str.is_empty() {
+                    Subdirectory::default()
+                } else {
+                    Subdirectory::try_from(subdir_str).unwrap_or_default()
+                };
                 SourceLocationSpec::Git(GitSpec {
                     git: git.clone(),
                     rev: rev.clone(),
-                    subdirectory: Some(relative_subdir.to_string()),
+                    subdirectory,
                 })
             }
         }
     }
 }
 
+/// Checks if a path's filename matches a known manifest file name.
+///
+/// This is used to determine if a path points to a file rather than a directory,
+/// without requiring filesystem access
+/// which wouldn't work for paths relative
+/// to the workspace root.
+fn is_known_manifest_file(path: Utf8TypedPath<'_>) -> bool {
+    path.file_name()
+        .map(|name| {
+            KNOWN_MANIFEST_FILES.contains(&name)
+                || RATTLER_BUILD_FILE_NAMES.contains(&name)
+                || ROS_BACKEND_FILE_NAMES.contains(&name)
+        })
+        .unwrap_or(false)
+}
+
 /// A slightly modified version of [`Utf8TypedPath::normalize`] that retains
 /// `..` components that lead outside the path.
 fn normalize_typed(path: Utf8TypedPath<'_>) -> Utf8TypedPathBuf {
