@@ -267,6 +267,7 @@ class SourceSpecTable(StrictBaseModel):
 
 MatchSpec = NonEmptyStr | MatchspecTable
 CondaPackageName = NonEmptyStr
+EnvironmentMarkers = NonEmptyStr
 
 
 class _PyPIRequirement(StrictBaseModel):
@@ -274,6 +275,10 @@ class _PyPIRequirement(StrictBaseModel):
         None,
         description="The [PEP 508 extras](https://peps.python.org/pep-0508/#extras) of the package",
     )
+    env_markers: EnvironmentMarkers | None = Field(
+        None,
+        description="The [PEP 508 markers](https://peps.python.org/pep-0508/#environment-markers) of the package",
+    )
 
 
 class _PyPiGitRequirement(_PyPIRequirement):
@@ -439,10 +444,13 @@ class TaskInlineTable(StrictBaseModel):
         description="A map of environment variables to values, used in the task, these will be overwritten by the shell.",
         examples=[{"key": "value"}, {"ARGUMENT": "value"}],
     )
+    default_environment: EnvironmentName | None = Field(
+        None,
+        description="A default environment to run the task",
+    )
     description: NonEmptyStr | None = Field(
         None,
         description="A short description of the task",
-        examples=["Build the project"],
     )
     clean_env: bool | None = Field(
         None,
